@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -52,13 +52,6 @@ ArpWidget::ArpWidget(MidiArp *p_midiWorker, GlobStore *p_globStore,
 
     midiControl = new MidiControl(this);
 
-    parStore = new ParStore(globStore, name, this);
-    midiControl->addMidiLearnMenu("Restore_"+name, parStore->topButton, 2);
-    connect(parStore, SIGNAL(store(int, bool)),
-             this, SLOT(storeParams(int, bool)));
-    connect(parStore, SIGNAL(restore(int)),
-             this, SLOT(restoreParams(int)));
-
     manageBox = new ManageBox("Arp:", false, this);
 
     // Input group box on left side
@@ -212,13 +205,24 @@ ArpWidget::ArpWidget(MidiArp *p_midiWorker, GlobStore *p_globStore,
             SLOT(selectPatternPreset(int)));
     midiControl->addMidiLearnMenu("PresetSwitch", patternPresetBox, 1);
 
-    muteOut = new QPushButton(tr("&Mute"),this);
+    muteOutAction = new QAction(tr("&Mute"),this);
+    muteOutAction->setCheckable(true);
+    connect(muteOutAction, SIGNAL(toggled(bool)), this, SLOT(setMuted(bool)));
+    muteOut = new QToolButton(this);
+    muteOut->setDefaultAction(muteOutAction);
     muteOut->setFont(QFont("Helvetica", 8));
-    muteOut->setMinimumSize(QSize(35,10));
-    muteOut->setCheckable(true);
-    connect(muteOut, SIGNAL(toggled(bool)), this, SLOT(setMuted(bool)));
+    muteOut->setMinimumSize(QSize(35,20));
     midiControl->addMidiLearnMenu("MuteToggle", muteOut, 0);
 
+    deferChangesAction = new QAction("D", this);
+    deferChangesAction->setToolTip(tr("Defer mute to pattern end"));
+    deferChangesAction->setCheckable(true);
+    connect(deferChangesAction, SIGNAL(toggled(bool)), this, SLOT(updateDeferChanges(bool)));
+
+    QToolButton *deferChangesButton = new QToolButton(this);
+    deferChangesButton->setDefaultAction(deferChangesAction);
+    deferChangesButton->setFixedSize(20, 20);
+
     repeatPatternThroughChord = new QComboBox(patternBox);
     QStringList repeatPatternNames;
     repeatPatternNames << tr("Static") << tr("Up") << tr("Down");
@@ -264,6 +268,7 @@ ArpWidget::ArpWidget(MidiArp *p_midiWorker, GlobStore *p_globStore,
     }
 
     modeLayout->addWidget(muteOut);
+    modeLayout->addWidget(deferChangesButton);
     modeLayout->addWidget(repeatPatternThroughChord);
     modeLayout->addWidget(triggerMode);
     modeLayout->addWidget(latchModeButton);
@@ -362,7 +367,15 @@ ArpWidget::ArpWidget(MidiArp *p_midiWorker, GlobStore *p_globStore,
     envelopeBox->setFlat(true);
     envelopeBox->setLayout(envelopeBoxLayout);
 
-    muteOut->setChecked(mutedAdd);
+
+    parStore = new ParStore(globStore, name, muteOutAction, deferChangesAction, this);
+    midiControl->addMidiLearnMenu("Restore_"+name, parStore->topButton, 2);
+    connect(parStore, SIGNAL(store(int, bool)),
+             this, SLOT(storeParams(int, bool)));
+    connect(parStore, SIGNAL(restore(int)),
+             this, SLOT(restoreParams(int)));
+
+    muteOutAction->setChecked(mutedAdd);
 
     QGridLayout *widgetLayout = new QGridLayout;
     widgetLayout->addWidget(patternBox, 0, 0);
@@ -389,6 +402,7 @@ MidiArp *ArpWidget::getMidiWorker()
 void ArpWidget::updateChIn(int value)
 {
     midiWorker->chIn = value;
+    modified = true;
 }
 
 void ArpWidget::updateIndexIn(int value)
@@ -399,6 +413,7 @@ void ArpWidget::updateIndexIn(int value)
         midiWorker->indexIn[1] = value;
     }
     checkIfInputFilterSet();
+    modified = true;
 }
 
 void ArpWidget::updateRangeIn(int value)
@@ -409,6 +424,7 @@ void ArpWidget::updateRangeIn(int value)
         midiWorker->rangeIn[1] = value;
     }
     checkIfInputFilterSet();
+    modified = true;
 }
 
 void ArpWidget::checkIfInputFilterSet()
@@ -455,6 +471,8 @@ void ArpWidget::writeData(QXmlStreamWriter& xml)
         xml.writeStartElement("output");
             xml.writeTextElement("muted", QString::number(
                 midiWorker->isMuted));
+            xml.writeTextElement("defer", QString::number(
+                midiWorker->deferChanges));
             xml.writeTextElement("port", QString::number(
                 midiWorker->portOut));
             xml.writeTextElement("channel", QString::number(
@@ -537,7 +555,9 @@ void ArpWidget::readData(QXmlStreamReader& xml)
                 if (xml.isEndElement())
                     break;
                 if (xml.name() == "muted")
-                    muteOut->setChecked(xml.readElementText().toInt());
+                    muteOutAction->setChecked(xml.readElementText().toInt());
+                else if (xml.name() == "defer")
+                    deferChangesAction->setChecked(xml.readElementText().toInt());
                 else if (xml.name() == "channel") {
                     tmp = xml.readElementText().toInt();
                     channelOut->setCurrentIndex(tmp);
@@ -826,7 +846,15 @@ void ArpWidget::setInOutBoxVisible(bool on)
 void ArpWidget::setMuted(bool on)
 {
     midiWorker->setMuted(on);
-    screen->setMuted(on);
+    screen->setMuted(midiWorker->isMuted);
+    parStore->ndc->setMuted(midiWorker->isMuted);
+    modified = true;
+}
+
+void ArpWidget::updateDeferChanges(bool on)
+{
+    midiWorker->updateDeferChanges(on);
+    modified = true;
 }
 
 void ArpWidget::setLatchMode(bool on)
@@ -893,28 +921,30 @@ void ArpWidget::storeParams(int ix, bool empty)
 
 void ArpWidget::restoreParams(int ix)
 {
+    midiWorker->applyPendingParChanges();
     if (parStore->list.at(ix).empty) return;
     patternText->setText(parStore->list.at(ix).pattern);
     repeatPatternThroughChord->setCurrentIndex(parStore->list.at(ix).repeatMode);
     updateRepeatPattern(parStore->list.at(ix).repeatMode);
-    indexIn[0]->setValue(parStore->list.at(ix).indexIn0);
-    indexIn[1]->setValue(parStore->list.at(ix).indexIn1);
-    rangeIn[0]->setValue(parStore->list.at(ix).rangeIn0);
-    rangeIn[1]->setValue(parStore->list.at(ix).rangeIn1);
-    attackTime->setValue(parStore->list.at(ix).attack);
-    releaseTime->setValue(parStore->list.at(ix).release);
-    randomTick->setValue(parStore->list.at(ix).rndTick);
-    randomLength->setValue(parStore->list.at(ix).rndLen);
-    randomVelocity->setValue(parStore->list.at(ix).rndVel);
-
-    //muteOut->setChecked(parStore->list.at(ix).muteOut);
-    chIn->setCurrentIndex(parStore->list.at(ix).chIn);
-    updateChIn(parStore->list.at(ix).chIn);
-    channelOut->setCurrentIndex(parStore->list.at(ix).channelOut);
-    updateChannelOut(parStore->list.at(ix).channelOut);
-    setPortOut(parStore->list.at(ix).portOut);
-    updatePortOut(parStore->list.at(ix).portOut);
+    if (!parStore->onlyPatternList.at(ix)) {
+        indexIn[0]->setValue(parStore->list.at(ix).indexIn0);
+        indexIn[1]->setValue(parStore->list.at(ix).indexIn1);
+        rangeIn[0]->setValue(parStore->list.at(ix).rangeIn0);
+        rangeIn[1]->setValue(parStore->list.at(ix).rangeIn1);
+        attackTime->setValue(parStore->list.at(ix).attack);
+        releaseTime->setValue(parStore->list.at(ix).release);
+        randomTick->setValue(parStore->list.at(ix).rndTick);
+        randomLength->setValue(parStore->list.at(ix).rndLen);
+        randomVelocity->setValue(parStore->list.at(ix).rndVel);
 
+        //muteOut->setChecked(parStore->list.at(ix).muteOut);
+        chIn->setCurrentIndex(parStore->list.at(ix).chIn);
+        updateChIn(parStore->list.at(ix).chIn);
+        channelOut->setCurrentIndex(parStore->list.at(ix).channelOut);
+        updateChannelOut(parStore->list.at(ix).channelOut);
+        setPortOut(parStore->list.at(ix).portOut);
+        updatePortOut(parStore->list.at(ix).portOut);
+    }
     midiWorker->advancePatternIndex(true);
 }
 
@@ -979,10 +1009,14 @@ void ArpWidget::updateDisplay()
     screen->updateDraw();
     midiControl->update();
 
-    if (!needsGUIUpdate) return;
+    if (!(needsGUIUpdate || midiWorker->needsGUIUpdate)) return;
 
     muteOut->setChecked(midiWorker->isMuted);
+    screen->setMuted(midiWorker->isMuted);
+    parStore->ndc->setMuted(midiWorker->isMuted);
     if (patternPresetBoxIndex != patternPresetBox->currentIndex())
         selectPatternPreset(patternPresetBoxIndex);
 
+    needsGUIUpdate = false;
+    midiWorker->needsGUIUpdate = false;
 }
