@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -111,7 +111,9 @@ class ArpWidget : public QWidget
     ManageBox *manageBox;
 
     QStringList patternPresets, patternNames;
-    QPushButton *muteOut;
+    QAction *muteOutAction;
+    QAction *deferChangesAction;
+    QToolButton *muteOut;
     QAction* hideInOutBoxAction;
 
     void setChIn(int value);
@@ -239,6 +241,15 @@ class ArpWidget : public QWidget
 */
     void setMuted(bool on);
 /*!
+* @brief Slot for the ArpWidget::deferChanges action.
+*
+* Sets a flag in the midi worker causing parameter changes to become
+* active/inactive only at pattern end.
+*
+* @param on Set to True for deferring parameter changes to pattern end
+*/
+    void updateDeferChanges(bool on);
+/*!
 * @brief This function stores some module parameters in a parameter
 * list object
 *
