@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -25,6 +25,8 @@
 
 #include <QString>
 
+#include <iostream>
+
 #include "seqdriver.h"
 #include "engine.h"
 
@@ -53,12 +55,12 @@ Engine::Engine(GlobStore *p_globStore, GrooveWidget *p_grooveWidget, int p_portC
     portCount = p_portCount;
 
     if (!p_alsamidi) {
-        driver = new JackDriver(portCount, this, tr_state_cb, midi_event_received_callback, tick_callback);
+        driver = new JackDriver(portCount, this, tr_state_cb, midi_event_received_callback, tick_callback, tempo_callback);
     }
     else {
     // In case of ALSA MIDI with Jack Transport sync, JackDriver is instantiated with 0 ports
     // a pointer to jackSync has to be passed to driver
-        jackSync = new JackDriver(0,  this, tr_state_cb, midi_event_received_callback, tick_callback);
+        jackSync = new JackDriver(0, this, tr_state_cb, midi_event_received_callback, tick_callback, tempo_callback);
         driver = new SeqDriver(jackSync, portCount, this, midi_event_received_callback, tick_callback);
     }
 
@@ -74,6 +76,7 @@ Engine::Engine(GlobStore *p_globStore, GrooveWidget *p_grooveWidget, int p_portC
     currentTick = 0;
     requestTick = 0;
     tempo = 120;
+    requestedTempo = 120;
 
     restoreRequest = -1;
     restoreModIx = 0;
@@ -82,6 +85,9 @@ Engine::Engine(GlobStore *p_globStore, GrooveWidget *p_grooveWidget, int p_portC
     restoreTick = -1;
     schedRestoreLocation = -1;
 
+    nextMinArpTick = 0;
+    nextMinLfoTick = 0;
+    nextMinSeqTick = 0;
     resetTicks(0);
     dispTimer = new MTimer();
     connect(dispTimer, SIGNAL(timeout()), this, SLOT(updateDisplay()));
@@ -453,8 +459,11 @@ void Engine::setStatus(bool on)
         }
     }
     status = on;
-    resetTicks(0);
     driver->setTransportStatus(on);
+    if (on) {
+        resetTicks(driver->getCurrentTick());
+        driver->requestEchoAt(0);
+    }
 }
 
 void Engine::tick_callback(void * context, bool echo_from_trig)
@@ -519,9 +528,9 @@ void Engine::echoCallback(bool echo_from_trig)
                 }
             }
             if (!l1)
-                nextMinLfoTick = midiLfo(l1)->nextTick;
-            else if (midiLfo(l1)->nextTick < nextMinLfoTick)
-                nextMinLfoTick = midiLfo(l1)->nextTick;
+                nextMinLfoTick = midiLfo(l1)->nextTick - schedDelayTicks;
+            else if (midiLfo(l1)->nextTick < nextMinLfoTick + schedDelayTicks)
+                nextMinLfoTick = midiLfo(l1)->nextTick - schedDelayTicks;
         }
         driver->requestEchoAt(nextMinLfoTick, 0);
     }
@@ -559,9 +568,9 @@ void Engine::echoCallback(bool echo_from_trig)
                 }
             }
             if (!l1)
-                nextMinSeqTick = midiSeq(l1)->nextTick;
-            else if (midiSeq(l1)->nextTick < nextMinSeqTick)
-                nextMinSeqTick = midiSeq(l1)->nextTick;
+                nextMinSeqTick = midiSeq(l1)->nextTick - schedDelayTicks;
+            else if (midiSeq(l1)->nextTick < nextMinSeqTick + schedDelayTicks)
+                nextMinSeqTick = midiSeq(l1)->nextTick - schedDelayTicks;
         }
         driver->requestEchoAt(nextMinSeqTick, 0);
     }
@@ -625,8 +634,8 @@ void Engine::echoCallback(bool echo_from_trig)
 
     if ((restoreTick > -1)
         && (!midiArpCount() || (nextMinArpTick + schedDelayTicks >= restoreTick))
-        && (!midiLfoCount() || (nextMinLfoTick >= restoreTick))
-        && (!midiSeqCount() || (nextMinSeqTick >= restoreTick))) {
+        && (!midiLfoCount() || (nextMinLfoTick + schedDelayTicks >= restoreTick))
+        && (!midiSeqCount() || (nextMinSeqTick + schedDelayTicks >= restoreTick))) {
         restoreTick = -1;
         restoreFlag = false;
         // TODO: At term the following should be restore() instead of
@@ -676,9 +685,6 @@ bool Engine::eventCallback(MidiEvent inEv)
     }
     for (l1 = 0; l1 < midiSeqCount(); l1++) {
         unmatched = midiSeq(l1)->handleEvent(inEv, tick);
-        if (inEv.type == EV_NOTEON && !unmatched && inEv.value) {
-            seqWidget(l1)->processNote(inEv.data, inEv.value);
-        }
         if (midiSeq(l1)->gotKbdTrig) {
             nextMinSeqTick = midiSeq(l1)->nextTick;
             no_collision = driver->requestEchoAt(nextMinSeqTick, true);
@@ -773,21 +779,26 @@ void Engine::resetTicks(int curtick)
     int l1;
 
     for (l1 = 0; l1 < midiArpCount(); l1++) {
-        midiArp(l1)->foldReleaseTicks(curtick);
-        midiArp(l1)->initArpTick(0);
-        midiArp(l1)->nextTick = 0;
+        midiArp(l1)->foldReleaseTicks(driver->trStartingTick - curtick);
+        midiArp(l1)->setNextTick(curtick);
+        if (!l1) nextMinArpTick = midiArp(l1)->nextTick;
+        if (midiArp(l1)->nextTick < nextMinArpTick)
+            nextMinArpTick=midiArp(l1)->nextTick;
     }
+
     for (l1 = 0; l1 < midiLfoCount(); l1++) {
-        midiLfo(l1)->setFramePtr(0);
-        midiLfo(l1)->nextTick = 0;
+        midiLfo(l1)->setNextTick(curtick);
+        if (!l1) nextMinLfoTick = midiLfo(l1)->nextTick;
+        if (midiLfo(l1)->nextTick < nextMinLfoTick)
+            nextMinLfoTick=midiLfo(l1)->nextTick;
     }
+
     for (l1 = 0; l1 < midiSeqCount(); l1++) {
-        midiSeq(l1)->setCurrentIndex(0);
-        midiSeq(l1)->nextTick = 0;
+        midiSeq(l1)->setNextTick(curtick);
+        if (!l1) nextMinSeqTick = midiSeq(l1)->nextTick;
+        if (midiSeq(l1)->nextTick < nextMinSeqTick)
+            nextMinSeqTick=midiSeq(l1)->nextTick;
     }
-    nextMinLfoTick = 0;
-    nextMinSeqTick = 0;
-    nextMinArpTick = 0;
 }
 
 void Engine::setMidiControllable(bool on)
@@ -830,13 +841,20 @@ void Engine::setMidiLearn(int moduleWindowID, int moduleID, int controlID)
     }
 }
 
-void Engine::setTempo(int bpm)
+void Engine::setTempo(double bpm)
 {
-    driver->setTempo(bpm);
+    driver->requestTempo(bpm);
+    requestedTempo = bpm;
+
     tempo=bpm;
     modified = true;
 }
 
+void Engine::tempo_callback(double bpm, void *context)
+{
+    ((Engine *)context)->requestedTempo = bpm;
+}
+
 void Engine::setSendLogEvents(bool on)
 {
     sendLogEvents = on;
@@ -964,6 +982,11 @@ void Engine::updateDisplay()
         }
         logEventCount = 0;
     }
+
+    if (requestedTempo != tempo) {
+        tempo = requestedTempo;
+        emit tempoUpdated(tempo);
+    }
 }
 
 MTimer::MTimer()
