@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -96,11 +96,20 @@ GlobStore::GlobStore(QWidget *parent)
 
     QToolButton *toolButton = new QToolButton(this);
     toolButton->setText("Global");
-    toolButton->setMinimumSize(QSize(50,30));
+    toolButton->setMinimumSize(QSize(48,40));
     midiControl->addMidiLearnMenu("GlobRestore", toolButton, 0);
+    QFrame *topRow = new QFrame(this);
+
+    QHBoxLayout *topRowLayout = new QHBoxLayout;
+    topRowLayout->addWidget(toolButton);
+    topRowLayout->setSpacing(0);
+    topRowLayout->setMargin(0);
+    topRow->setFrameStyle(QFrame::StyledPanel);
+    topRow->setMinimumSize(QSize(48,46));;
+    topRow->setLayout(topRowLayout);
 
     QVBoxLayout *buttonLayout = new QVBoxLayout;
-    buttonLayout->addWidget(toolButton);
+    buttonLayout->addWidget(topRow);
 
     QVBoxLayout *columnLayout = new QVBoxLayout;
     columnLayout->addLayout(buttonLayout);
@@ -115,7 +124,7 @@ GlobStore::GlobStore(QWidget *parent)
 
     QHBoxLayout *secondRowLayout = new QHBoxLayout;
     secondRowLayout->addLayout(indivButtonLayout);
-    secondRowLayout->addStretch();
+    secondRowLayout->addStretch(1);
 
     QVBoxLayout *centLayout = new QVBoxLayout();
     centLayout->addLayout(upperRowLayout);
