@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -32,23 +32,31 @@ JackDriver::JackDriver(
     void * callback_context,
     void (* p_tr_state_cb)(bool j_tr_state, void * context),
     bool (* midi_event_received_callback)(void * context, MidiEvent ev),
-    void (* tick_callback)(void * context, bool echo_from_trig))
+    void (* tick_callback)(void * context, bool echo_from_trig),
+    void (* p_tempo_callback)(double bpm, void * context))
     : DriverBase(callback_context, midi_event_received_callback, tick_callback, 60e9)
 {
     portCount = p_portCount;
     cbContext = callback_context;
     trStateCb = p_tr_state_cb;
+    tempoCb = p_tempo_callback;
     jackRunning = false;
     portUnmatched = 0;
     forwardUnmatched = false;
     useJackSync = false;
-    echoTickQueue.resize(512);
-    evQueue.resize(512);
-    evTickQueue.resize(512);
-    evPortQueue.resize(512);
+    echoTickQueue.resize(JQ_BUFSZ);
+    evQueue.resize(JQ_BUFSZ);
+    evTickQueue.resize(JQ_BUFSZ);
+    evPortQueue.resize(JQ_BUFSZ);
     bufPtr = 0;
     echoPtr = 0;
     internalTempo = 120;
+    requestedTempo = 120;
+    tempoChangeTick = 0;
+    tempoChangeJPosFrame = 0;
+    jackNFrames = 256;
+    trStartingTick = 0;
+    trLoopingTick = 0;
 
 /* Initialize and activate Jack with out_port_count ports if we use
  *  JACK driver backend, i.e. portCount > 0 */
@@ -59,7 +67,7 @@ JackDriver::JackDriver(
         transportState = JackTransportStopped;
     }
 }
-bool JackDriver::callJack(int port_count)
+bool JackDriver::callJack(int port_count, const QString & clientname)
 {
     if (port_count == -1 && jackRunning) {
         deactivateJack();
@@ -69,7 +77,7 @@ bool JackDriver::callJack(int port_count)
         }
     }
     else if (port_count != -1) {
-        if (initJack(port_count)) {
+        if (initJack(port_count, clientname)) {
             emit j_shutdown();
             return true;
         }
@@ -94,23 +102,23 @@ JackDriver::~JackDriver()
     }
 }
 
-int JackDriver::initJack(int out_port_count)
+int JackDriver::initJack(int out_port_count, const QString & clientname)
 {
     char buf[16];
 
 #ifdef JACK_SESSION
        if (global_jack_session_uuid.isEmpty() || !out_port_count) {
-           if ((jack_handle = jack_client_open(PACKAGE, JackNullOption, NULL)) == 0) {
+           if ((jack_handle = jack_client_open(clientname.toLatin1(), JackNullOption, NULL)) == 0) {
             qCritical("jack server not running?");
                return 1;
            }
        }
-       else if ((jack_handle = jack_client_open(PACKAGE, JackSessionID, NULL, global_jack_session_uuid.data())) == 0) {
+       else if ((jack_handle = jack_client_open(clientname.toLatin1(), JackSessionID, NULL, global_jack_session_uuid.data())) == 0) {
         qCritical("jack server not running?");
            return 1;
        }
 #else
-    if ((jack_handle = jack_client_open(PACKAGE, JackNullOption, NULL)) == 0) {
+    if ((jack_handle = jack_client_open(clientname.toLatin1(), JackNullOption, NULL)) == 0) {
         qCritical("jack server not running?");
         return 1;
     }
@@ -194,11 +202,13 @@ int JackDriver::process_callback(jack_nframes_t nframes, void *arg)
 
     if (!out_port_count) return (0);
 
+    rd->handleEchoes(nframes);
+
     int cur_tempo = rd->tempo;
     uint64_t cur_j_frame = rd->curJFrame;
     bool forward_unmatched = rd->forwardUnmatched;
     int port_unmatched = rd->portUnmatched;
-
+    uint tempochangetick = rd->tempoChangeTick;
     uint nexttick = 0;
     uint tmptick = 0;
     uint idx = 0;
@@ -213,7 +223,6 @@ int JackDriver::process_callback(jack_nframes_t nframes, void *arg)
     MidiEvent outEv;
     outEv.channel = 0;
 
-
     unsigned char* buffer;
     jack_midi_event_t in_event;
     jack_nframes_t event_index = 0;
@@ -233,7 +242,6 @@ int JackDriver::process_callback(jack_nframes_t nframes, void *arg)
     for(i = 0; i < nframes; i++) {
 
         /* MIDI Output queue first **/
-        //size = rd->evTickQueue.size();
         if (rd->bufPtr) { /* If we have events, find earliest event tick **/
             idx = 0;
             nexttick = rd->evTickQueue.first();
@@ -244,11 +252,11 @@ int JackDriver::process_callback(jack_nframes_t nframes, void *arg)
                     nexttick = tmptick;
                 }
             }
-            ev_sample = (uint64_t)j_sample_rate * 60 * nexttick / (TPQN * cur_tempo);
+            ev_sample = (uint64_t)j_sample_rate * 60 * (nexttick - tempochangetick) / TPQN / cur_tempo;
             ev_jframe = ev_sample / nframes;
             ev_inframe = ev_sample % nframes;
             if ((ev_jframe <= cur_j_frame) && (ev_inframe <= i)) {
-                //qWarning("nexttick %d, ev_frame %d, ev_inframe %d, cur_jframe %d", nexttick, ev_jframe, ev_inframe, cur_j_frame);
+                //qWarning("nexttick %d, ev_frame %d, ev_inframe %d, cur_jframe %d, buf_idx %d", nexttick, ev_jframe, ev_inframe, cur_j_frame, idx);
                 outEv = rd->evQueue.at(idx);
                 evport = rd->evPortQueue.at(idx);
                 for (uint l4 = idx ; l4 < (rd->bufPtr - 1);l4++) {
@@ -258,21 +266,17 @@ int JackDriver::process_callback(jack_nframes_t nframes, void *arg)
                 }
                 rd->bufPtr--;
                 int k = 0;
-                if ((ev_jframe) <= cur_j_frame) {
-                    do {
-                        if ((ev_jframe) < cur_j_frame) {
-                            ev_inframe = 0;
-                        }
-                        buffer = jack_midi_event_reserve(out_buf[evport], ev_inframe + k, 3);
-                        k++;
-                    } while (buffer == NULL);
+                do {
+                    if ((ev_jframe) < cur_j_frame) ev_inframe = 0;
+                    buffer = jack_midi_event_reserve(out_buf[evport], ev_inframe + k, 3);
+                    k++;
+                } while (buffer == NULL);
 
-                    buffer[2] = outEv.value;        /* velocity / value **/
-                    buffer[1] = outEv.data;         /* note / controller **/
-                    if (outEv.type == EV_NOTEON) buffer[0] = 0x90;
-                    if (outEv.type == EV_CONTROLLER) buffer[0] = 0xb0;
-                    buffer[0] += outEv.channel;
-                }
+                buffer[2] = outEv.value;        /* velocity / value **/
+                buffer[1] = outEv.data;         /* note / controller **/
+                if (outEv.type == EV_NOTEON) buffer[0] = 0x90;
+                if (outEv.type == EV_CONTROLLER) buffer[0] = 0xb0;
+                buffer[0] += outEv.channel;
             }
         }
         /* MIDI Input handling **/
@@ -328,7 +332,7 @@ int JackDriver::process_callback(jack_nframes_t nframes, void *arg)
                 jack_midi_event_get(&in_event, in_buf, event_index);
         }
     }
-    rd->handleEchoes(nframes);
+    rd->curJFrame++;
     return(0);
 }
 
@@ -369,6 +373,10 @@ void JackDriver::jackTrCheckState()
 
     uint state = getState();
 
+    if (!portCount && (currentPos.beats_per_minute != tempo)) {
+        tempoCb(currentPos.beats_per_minute, cbContext);
+    }
+
     if (transportState == state) return;
     transportState = state;
     switch (state){
@@ -383,10 +391,12 @@ void JackDriver::jackTrCheckState()
         break;
 
         case JackTransportStarting:
+            trStartingTick = m_current_tick;
             printf( "[JackTransportStarting]\n" );
         break;
 
         case JackTransportLooping:
+            trLoopingTick = m_current_tick;
             printf( "[JackTransportLooping]\n" );
         break;
         default:
@@ -413,6 +423,10 @@ void JackDriver::sendMidiEvent(MidiEvent ev, int n_tick, unsigned outport, unsig
 {
   //qWarning("sendMidiEvent([%d, %d, %d, %d], %u, %u) at tick %d", ev.type, ev.channel, ev.data, ev.value, outport, duration, n_tick);
 
+    if (bufPtr > JQ_BUFSZ - 2) {
+        printf("WARNING: Event buffer overflow. Buffer cleared.\n");
+        bufPtr = 0;
+    }
     evQueue.replace(bufPtr,ev);
     evTickQueue.replace(bufPtr,n_tick);
     evPortQueue.replace(bufPtr,outport);
@@ -429,6 +443,10 @@ void JackDriver::sendMidiEvent(MidiEvent ev, int n_tick, unsigned outport, unsig
 
 bool JackDriver::requestEchoAt(int echo_tick, bool echo_from_trig)
 {
+    if (echoPtr > JQ_BUFSZ - 1) {
+        printf("WARNING: Echo buffer overflow. Buffer cleared.\n");
+        echoPtr = 0;
+    }
     if ((echo_tick == (int)lastSchedTick) && (echo_tick)) return false;
     echoTickQueue.replace(echoPtr, echo_tick);
     echoPtr++;
@@ -441,22 +459,33 @@ bool JackDriver::requestEchoAt(int echo_tick, bool echo_from_trig)
 
 void JackDriver::handleEchoes(int nframes)
 {
-    curJFrame++;
-
-    if (!queueStatus) return;
-
     uint l1;
     int nexttick, tmptick, idx;
 
+    jackNFrames = nframes;
     if (useJackSync) {
-        m_current_tick = ((uint64_t)currentPos.frame * TPQN * tempo
-            / (currentPos.frame_rate * 60)) - jackOffsetTick;
+        if (currentPos.beats_per_minute > 0.01) {
+        m_current_tick = ((uint64_t)currentPos.frame - tempoChangeJPosFrame)
+            * TPQN * tempo / (currentPos.frame_rate * 60) + tempoChangeTick;
+            if (currentPos.beats_per_minute != tempo) {
+                tempoChangeJPosFrame = currentPos.frame;
+                setTempo(currentPos.beats_per_minute);
+                // inform engine via callback about the tempo change
+                tempoCb(tempo, cbContext);
+                requestedTempo = currentPos.beats_per_minute;
+            }
+        }
     }
     else {
-        m_current_tick = (uint64_t)curJFrame * TPQN * tempo * nframes
-            / (jSampleRate * 60);
+        m_current_tick =  (uint64_t)curJFrame * TPQN * tempo * nframes
+            / (jSampleRate * 60) + tempoChangeTick;
+        if (requestedTempo != tempo) setTempo(requestedTempo);
     }
 
+    if (!queueStatus) {
+        curJFrame++;
+        return;
+    }
     if (!echoPtr) return;
 
     idx = 0;
@@ -478,31 +507,43 @@ void JackDriver::handleEchoes(int nframes)
     }
 }
 
+void JackDriver::setTempo(double bpm)
+{
+    tempoChangeTick = m_current_tick;
+    curJFrame = 0;
+    tempo = bpm;
+    internalTempo = bpm;
+}
+
 void JackDriver::setTransportStatus(bool on)
 {
     jack_position_t jpos = getCurrentPos();
     if (useJackSync) {
         if (jpos.beats_per_minute > 0.01)
-            tempo = (int)jpos.beats_per_minute;
+            tempo = jpos.beats_per_minute;
         else
             tempo = internalTempo;
 
-        jackOffsetTick = (uint64_t)jpos.frame * TPQN
-            * tempo / (jpos.frame_rate * 60);
+        tempoChangeJPosFrame = 0;
+        tempoChangeTick = 0;
     }
     else {
         tempo = internalTempo;
-        jackOffsetTick = 0;
     }
 
-    m_current_tick = 0;
-
     if (on) {
-        curJFrame = 0;
+        if (useJackSync) {
+            m_current_tick = ((uint64_t)currentPos.frame - tempoChangeJPosFrame)
+                * TPQN * tempo / (currentPos.frame_rate * 60) + tempoChangeTick;
+            curJFrame = ((uint64_t)currentPos.frame - tempoChangeJPosFrame) * (jSampleRate * 60) / jackNFrames / (currentPos.frame_rate * 60);
+        } else {
+            m_current_tick = 0;
+            curJFrame = 0;
+        }
+        tempoChangeTick = 0;
         lastSchedTick = 0;
         echoPtr = 0;
         bufPtr = 0;
-        requestEchoAt(0);
         printf("Internal Transport started\n");
     }
     else {
