@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -66,9 +66,11 @@ class JackDriver : public DriverBase
 
     bool jackRunning;
     uint transportState;
+    uint jackNFrames;
     uint lastSchedTick;
-    uint jackOffsetTick;
+    uint tempoChangeTick;
     uint64_t curJFrame;
+    uint64_t tempoChangeJPosFrame;
     QVector<uint> echoTickQueue;
     QVector<MidiEvent> evQueue;
     QVector<uint> evTickQueue;
@@ -91,10 +93,12 @@ class JackDriver : public DriverBase
             void * callback_context,
             void (* p_tr_state_cb)(bool j_tr_state, void * context),
             bool (* midi_event_received_callback)(void * context, MidiEvent ev),
-            void (* tick_callback)(void * context, bool echo_from_trig));
+            void (* tick_callback)(void * context, bool echo_from_trig),
+            void (* p_tempo_callback)(double bpm, void * context));
     ~JackDriver();
 
     void (* trStateCb)(bool j_tr_state, void * context);
+    void (* tempoCb)(double bpm, void * context);
     void * cbContext;
 
   signals:
@@ -104,7 +108,7 @@ class JackDriver : public DriverBase
   public:
     jack_nframes_t jSampleRate;
     bool isRunning() { return jackRunning; }
-    int initJack(int out_port_count);
+    int initJack(int out_port_count, const QString & clientname=PACKAGE);
     int activateJack();
     int deactivateJack();
 
@@ -116,8 +120,9 @@ class JackDriver : public DriverBase
     jack_position_t getCurrentPos();
     bool requestEchoAt(int echoTick, bool echo_from_trig = 0);
     void setTransportStatus(bool run);
+    void setTempo(double bpm);
     int getClientId() {return 0; }
-    bool callJack(int portcount);
+    bool callJack(int portcount, const QString & clientname=PACKAGE);
 };
 
 
