@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -86,7 +86,7 @@ class LfoScreen : public QWidget
     void updateData(const QVector<Sample>& data);
     void mouseMoveEvent(QMouseEvent* event);
     void mousePressEvent(QMouseEvent* event);
-    void setRecord(bool on);
+    void setRecordMode(bool on);
     void wheelEvent(QWheelEvent* event);
     void newGrooveValues(int tick, int vel, int length);
     void setMuted(bool on);
