@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -53,13 +53,6 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, GlobStore *p_globStore,
 
     midiControl = new MidiControl(this);
 
-    parStore = new ParStore(globStore, name, this);
-    midiControl->addMidiLearnMenu("Restore_"+name, parStore->topButton, 9);
-    connect(parStore, SIGNAL(store(int, bool)),
-             this, SLOT(storeParams(int, bool)));
-    connect(parStore, SIGNAL(restore(int)),
-             this, SLOT(restoreParams(int)));
-
     manageBox = new ManageBox("LFO:", true, this);
 
     // Input group box on right top
@@ -290,13 +283,23 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, GlobStore *p_globStore,
             SLOT(updateLoop(int)));
     midiControl->addMidiLearnMenu("LoopMode", loopBox, 8);
 
-    muteOut = new QPushButton(tr("&Mute"),this);
+    muteOutAction = new QAction(tr("&Mute"),this);
+    muteOutAction->setCheckable(true);
+    connect(muteOutAction, SIGNAL(toggled(bool)), this, SLOT(setMuted(bool)));
+    muteOut = new QToolButton(this);
+    muteOut->setDefaultAction(muteOutAction);
     muteOut->setFont(QFont("Helvetica", 8));
-    muteOut->setMinimumSize(QSize(35,10));
-    muteOut->setCheckable(true);
-    connect(muteOut, SIGNAL(toggled(bool)), this, SLOT(setMuted(bool)));
+    muteOut->setMinimumSize(QSize(35,20));
     midiControl->addMidiLearnMenu("MuteToggle", muteOut, 0);
 
+    deferChangesAction = new QAction("D", this);
+    deferChangesAction->setToolTip(tr("Defer mute to pattern end"));
+    deferChangesAction->setCheckable(true);
+    connect(deferChangesAction, SIGNAL(toggled(bool)), this, SLOT(updateDeferChanges(bool)));
+
+    QToolButton *deferChangesButton = new QToolButton(this);
+    deferChangesButton->setDefaultAction(deferChangesAction);
+    deferChangesButton->setFixedSize(20, 20);
 
     QLabel *recordButtonLabel = new QLabel(tr("Re&cord"), waveBox);
     recordAction = new QAction(QIcon(seqrecord_xpm), tr("Re&cord"), waveBox);
@@ -332,7 +335,8 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, GlobStore *p_globStore,
 
     QGridLayout *paramBoxLayout = new QGridLayout;
     paramBoxLayout->addWidget(loopBox, 0, 0, 1, 2);
-    paramBoxLayout->addWidget(muteOut, 1, 0, 1, 2);
+    paramBoxLayout->addWidget(muteOut, 1, 0, 1, 1);
+    paramBoxLayout->addWidget(deferChangesButton, 1, 1, 1, 2);
     paramBoxLayout->addWidget(recordButtonLabel, 2, 0);
     paramBoxLayout->addWidget(recordButton, 2, 1);
     paramBoxLayout->addWidget(waveFormBoxLabel, 0, 2);
@@ -361,13 +365,21 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, GlobStore *p_globStore,
     }
     waveBox->setLayout(waveBoxLayout);
 
-    muteOut->setChecked(mutedAdd);
 
     QHBoxLayout *widgetLayout = new QHBoxLayout;
     widgetLayout->addWidget(waveBox, 1);
     widgetLayout->addWidget(hideInOutBoxButton, 0);
     widgetLayout->addWidget(inOutBox, 0);
 
+    parStore = new ParStore(globStore, name, muteOutAction, deferChangesAction, this);
+    midiControl->addMidiLearnMenu("Restore_"+name, parStore->topButton, 9);
+    connect(parStore, SIGNAL(store(int, bool)),
+             this, SLOT(storeParams(int, bool)));
+    connect(parStore, SIGNAL(restore(int)),
+             this, SLOT(restoreParams(int)));
+
+    muteOutAction->setChecked(mutedAdd);
+
     setLayout(widgetLayout);
     updateAmp(64);
 
@@ -412,6 +424,8 @@ void LfoWidget::writeData(QXmlStreamWriter& xml)
         xml.writeStartElement("output");
             xml.writeTextElement("muted", QString::number(
                 midiWorker->isMuted));
+            xml.writeTextElement("defer", QString::number(
+                midiWorker->deferChanges));
             xml.writeTextElement("port", QString::number(
                 midiWorker->portOut));
             xml.writeTextElement("channel", QString::number(
@@ -505,7 +519,9 @@ void LfoWidget::readData(QXmlStreamReader& xml)
                 if (xml.isEndElement())
                     break;
                 if (xml.name() == "muted")
-                    muteOut->setChecked(xml.readElementText().toInt());
+                    muteOutAction->setChecked(xml.readElementText().toInt());
+                else if (xml.name() == "defer")
+                    deferChangesAction->setChecked(xml.readElementText().toInt());
                 else if (xml.name() == "channel") {
                     tmp = xml.readElementText().toInt();
                     channelOut->setCurrentIndex(tmp);
@@ -667,7 +683,7 @@ void LfoWidget::updateWaveForm(int val)
     midiWorker->getData(&data);
     screen->updateData(data);
     bool isCustom = (val == 5);
-    if (isCustom) newCustomOffset();
+    if (isCustom) midiWorker->newCustomOffset();
     amplitude->setDisabled(isCustom);
     freqBox->setDisabled(isCustom);
     modified = true;
@@ -697,7 +713,7 @@ void LfoWidget::updateRes(int val)
     midiWorker->updateResolution(lfoResValues[val]);
     midiWorker->getData(&data);
     screen->updateData(data);
-    newCustomOffset();
+    midiWorker->newCustomOffset();
     modified = true;
 }
 
@@ -708,7 +724,7 @@ void LfoWidget::updateSize(int val)
     midiWorker->updateSize(sizeBox->currentText().toInt());
     midiWorker->getData(&data);
     screen->updateData(data);
-    newCustomOffset();
+    midiWorker->newCustomOffset();
     modified = true;
 }
 
@@ -743,20 +759,6 @@ void LfoWidget::copyToCustom()
     modified = true;
 }
 
-void LfoWidget::newCustomOffset()
-{
-    int min = 127;
-    int value;
-    const int npoints = sizeBox->currentText().toInt()
-                        * resBox->currentText().toInt();
-    for (int l1 = 0; l1 < npoints; l1++) {
-        value = data.at(l1).value;
-        if (value < min) min = value;
-    }
-    midiWorker->cwmin = min;
-    offset->setValue(min);
-}
-
 void LfoWidget::mouseMoved(double mouseX, double mouseY, int buttons)
 {
     if (buttons == 2) {
@@ -767,10 +769,9 @@ void LfoWidget::mouseMoved(double mouseX, double mouseY, int buttons)
             copyToCustom();
         }
         midiWorker->setCustomWavePoint(mouseX, mouseY, false);
-        newCustomOffset();
+        midiWorker->newCustomOffset();
     }
-    midiWorker->getData(&data);
-    screen->updateData(data);
+    midiWorker->dataChanged = true;
     modified = true;
 }
 
@@ -784,10 +785,9 @@ void LfoWidget::mousePressed(double mouseX, double mouseY, int buttons)
             copyToCustom();
         }
         midiWorker->setCustomWavePoint(mouseX, mouseY, true);
-        newCustomOffset();
+        midiWorker->newCustomOffset();
     }
-    midiWorker->getData(&data);
-    screen->updateData(data);
+    midiWorker->dataChanged = true;
     modified = true;
 }
 
@@ -808,17 +808,21 @@ void LfoWidget::setInOutBoxVisible(bool on)
 void LfoWidget::setMuted(bool on)
 {
     midiWorker->setMuted(on);
-    screen->setMuted(on);
+    screen->setMuted(midiWorker->isMuted);
+    parStore->ndc->setMuted(midiWorker->isMuted);
+    modified = true;
+}
+
+void LfoWidget::updateDeferChanges(bool on)
+{
+    midiWorker->updateDeferChanges(on);
+    modified = true;
 }
 
 void LfoWidget::setRecord(bool on)
 {
-    if (!on) {
-        midiWorker->isRecording = false;
-        newCustomOffset();
-    }
-    midiWorker->recordMode = on;
-    screen->setRecord(on);
+    midiWorker->setRecordMode(on);
+    screen->setRecordMode(on);
 }
 
 void LfoWidget::setPortOut(int value)
@@ -881,6 +885,7 @@ void LfoWidget::storeParams(int ix, bool empty)
 
 void LfoWidget::restoreParams(int ix)
 {
+    midiWorker->applyPendingParChanges();
     if (parStore->list.at(ix).empty) return;
     for (int l1 = 0; l1 < parStore->list.at(ix).wave.count(); l1++) {
         midiWorker->customWave.replace(l1, parStore->list.at(ix).wave.at(l1));
@@ -896,20 +901,21 @@ void LfoWidget::restoreParams(int ix)
     resBox->setCurrentIndex(parStore->list.at(ix).res);
     waveFormBox->setCurrentIndex(parStore->list.at(ix).waveForm);
     loopBox->setCurrentIndex(parStore->list.at(ix).loopMode);
-
-    //muteOut->setChecked(parStore->list.at(ix).muteOut);
-    chIn->setCurrentIndex(parStore->list.at(ix).chIn);
-    updateChIn(parStore->list.at(ix).chIn);
-    ccnumberInBox->setValue(parStore->list.at(ix).ccnumberIn);
-    ccnumberBox->setValue(parStore->list.at(ix).ccnumber);
-    channelOut->setCurrentIndex(parStore->list.at(ix).channelOut);
-    updateChannelOut(parStore->list.at(ix).channelOut);
-    setPortOut(parStore->list.at(ix).portOut);
-    updatePortOut(parStore->list.at(ix).portOut);
     updateLoop(parStore->list.at(ix).loopMode);
     updateWaveForm(parStore->list.at(ix).waveForm);
-    offset->setValue(parStore->list.at(ix).offs);
-    amplitude->setValue(parStore->list.at(ix).ampl);
+    if (!parStore->onlyPatternList.at(ix)) {
+        //muteOut->setChecked(parStore->list.at(ix).muteOut);
+        offset->setValue(parStore->list.at(ix).offs);
+        chIn->setCurrentIndex(parStore->list.at(ix).chIn);
+        updateChIn(parStore->list.at(ix).chIn);
+        ccnumberInBox->setValue(parStore->list.at(ix).ccnumberIn);
+        ccnumberBox->setValue(parStore->list.at(ix).ccnumber);
+        channelOut->setCurrentIndex(parStore->list.at(ix).channelOut);
+        updateChannelOut(parStore->list.at(ix).channelOut);
+        setPortOut(parStore->list.at(ix).portOut);
+        updatePortOut(parStore->list.at(ix).portOut);
+        amplitude->setValue(parStore->list.at(ix).ampl);
+    }
     midiWorker->setFramePtr(0);
 }
 
@@ -960,7 +966,7 @@ void LfoWidget::copyParamsFrom(LfoWidget *fromWidget)
         midiWorker->muteMask.replace(l1, midiWorker->customWave.at(l1).muted);
     }
     midiControl->setCcList(fromWidget->midiControl->ccList);
-    muteOut->setChecked(true);
+    muteOutAction->setChecked(true);
 
     tmp = fromWidget->waveFormBox->currentIndex();
     waveFormBox->setCurrentIndex(tmp);
@@ -1020,16 +1026,16 @@ void LfoWidget::handleController(int ccnumber, int channel, int value)
                 case 5: if (min == max) {
                             if (value == max) {
                                 m = midiWorker->recordMode;
-                                setRecord(!m);
+                                midiWorker->setRecordMode(!m);
                                 return;
                             }
                         }
                         else {
                             if (value == max) {
-                                setRecord(true);
+                                midiWorker->setRecordMode(true);
                             }
                             if (value == min) {
-                                setRecord(false);
+                                midiWorker->setRecordMode(false);
                             }
                         }
                 break;
@@ -1076,16 +1082,20 @@ void LfoWidget::updateDisplay()
         midiWorker->getData(&data);
         screen->updateData(data);
         cursor->updateNumbers(midiWorker->res, midiWorker->size);
+        offset->setValue(midiWorker->offs);
         midiWorker->dataChanged = false;
     }
     screen->updateDraw();
     cursor->updateDraw();
     midiControl->update();
 
-    if (!needsGUIUpdate) return;
+    if (!(needsGUIUpdate || midiWorker->needsGUIUpdate)) return;
 
-    muteOut->setChecked(midiWorker->isMuted);
+    muteOutAction->setChecked(midiWorker->isMuted);
+    screen->setMuted(midiWorker->isMuted);
+    parStore->ndc->setMuted(midiWorker->isMuted);
     recordAction->setChecked(midiWorker->recordMode);
+    screen->setRecordMode(midiWorker->recordMode);
     resBox->setCurrentIndex(resBoxIndex);
     updateRes(resBoxIndex);
     sizeBox->setCurrentIndex(sizeBoxIndex);
@@ -1100,4 +1110,5 @@ void LfoWidget::updateDisplay()
         updateWaveForm(waveFormBoxIndex);
     }
     needsGUIUpdate = false;
+    midiWorker->needsGUIUpdate = false;
 }
