@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -88,16 +88,6 @@ class LfoWidget : public QWidget
 */
     void loadWaveForms();
 
-/*!
-* @brief This function determines the minimum of the current waveform and
-* sets the LfoWidget::offset slider accordingly.
-*
-* It also sets MidiLfo::cwmin. When a new waveform is drawn, its minimum
-* offset from 0 changes and the offset controller has to be adapted in range.
-*
-*/
-    void newCustomOffset();
-
 /* PUBLIC MEMBERS */
   public:
 /*!
@@ -130,7 +120,6 @@ class LfoWidget : public QWidget
     QComboBox *resBox, *sizeBox;
     QComboBox *loopBox;
     QWidget *inOutBox;
-    QPushButton *muteOut;
     QCheckBox *enableNoteOff;
     QCheckBox *enableRestartByKbd;
     QCheckBox *enableTrigByKbd;
@@ -138,6 +127,9 @@ class LfoWidget : public QWidget
     Slider *frequency, *amplitude, *offset;
     QAction *recordAction;
     QAction* hideInOutBoxAction;
+    QAction *muteOutAction;
+    QAction *deferChangesAction;
+    QToolButton *muteOut;
     QComboBox *waveFormBox, *freqBox;
 
     void setChIn(int value);
@@ -371,6 +363,15 @@ class LfoWidget : public QWidget
 */
     void setMuted(bool on);
 /*!
+* @brief Slot for the LfoWidget::deferChanges action.
+*
+* Sets a flag in the midi worker causing parameter changes to become
+* active/inactive only at pattern end.
+*
+* @param on Set to True for deferring parameter changes to pattern end
+*/
+    void updateDeferChanges(bool on);
+/*!
 * @brief This function stores some module parameters in a parameter
 * list object
 *
