@@ -10,7 +10,7 @@
  *
  *   @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -149,7 +149,7 @@ int main(int argc, char *argv[])
         app.installTranslator(&qmidiarpTr);
 #endif
 
-    MainWindow* qmidiarp = new MainWindow(portCount, alsamidi);
+    MainWindow* qmidiarp = new MainWindow(portCount, alsamidi, argv[0]);
     if (optind < argc) {
         QFileInfo fi(argv[optind]);
         if (fi.exists())
