@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@
 #include <cstring>  // for strerror()
 #include <unistd.h> // for pipe()
 
+#include <iostream>
 
 #include "mainwindow.h"
 
@@ -72,9 +73,15 @@
 static const char FILEEXT[] = ".qmax";
 
 int MainWindow::sigpipe[2];
+#ifdef NSM
+nsm_client_t *MainWindow::nsm = 0;
+#endif
 
-MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
+MainWindow::MainWindow(int p_portCount, bool p_alsamidi, char *execName)
 {
+#ifndef NSM
+(void)execName;
+#endif
     jackFailed = false;
     filename = "";
     lastDir = QDir::homePath();
@@ -98,6 +105,33 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
     globStoreWindow->setObjectName("globStore");
     globStoreWindow->setVisible(true);
 
+#ifdef NSM
+    const char *nsm_url = getenv( "NSM_URL" );
+
+    if ( nsm_url )
+    {
+        nsm = nsm_new();
+
+        nsm_set_open_callback( nsm, cb_nsm_open, this );
+        nsm_set_save_callback( nsm, cb_nsm_save, this );
+
+        if ( 0 == nsm_init_thread( nsm, nsm_url ) )
+        {
+            connect(this, SIGNAL(nsmOpenFile(const QString &)), this,
+                    SLOT(openFile(const QString &)));
+            nsm_send_announce( nsm, APP_NAME, ":switch:", execName );
+            nsm_thread_start(nsm);
+        }
+        else
+        {
+            nsm_free( nsm );
+            nsm = 0;
+        }
+    }
+#else
+    bool nsm = 0;
+#endif
+
     engine = new Engine(globStore, grooveWidget, p_portCount, alsaMidi, this);
     if (alsaMidi) {
         connect(engine->jackSync, SIGNAL(j_shutdown()), this, SLOT(jackShutdown()));
@@ -105,8 +139,11 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
     else {
         connect(engine->driver, SIGNAL(jsEvent(int)), this, SLOT(jsAction(int)));
         connect(engine->driver, SIGNAL(j_shutdown()), this, SLOT(jackShutdown()));
-        if (engine->driver->callJack(p_portCount)) jackFailed = true;
+        if (!nsm && engine->driver->callJack(p_portCount, PACKAGE)) jackFailed = true;
     }
+    connect(engine, SIGNAL(tempoUpdated(double)), this,
+            SLOT(displayTempo(double)));
+
     connect(globStore, SIGNAL(store(int)), engine,
             SLOT(store(int)));
     connect(globStore, SIGNAL(requestRestore(int)), engine,
@@ -342,6 +379,16 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
         qWarning("%s", "Signal handlers not installed!");
 
     if (!jackFailed || alsaMidi) show();
+
+#ifdef NSM
+    if (nsm && nsm_is_active(nsm))
+    {
+        fileNewAction->setDisabled(true);
+        fileOpenAction->setDisabled(true);
+        fileSaveAsAction->setDisabled(true);
+        fileRecentlyOpenedFiles->setDisabled(true);
+    }
+#endif
 }
 
 MainWindow::~MainWindow()
@@ -697,9 +744,20 @@ void MainWindow::openFile(const QString& fn)
 
     QFile f(fn);
     if (!f.open(QIODevice::ReadOnly)) {
-        QMessageBox::warning(this, APP_NAME,
+#ifdef NSM
+        if (nsm_is_active(nsm)) {
+            filename = fn;
+            //updateWindowTitle();
+            return;
+        }
+        else {
+#endif
+            QMessageBox::warning(this, APP_NAME,
                 tr("Could not read from file '%1'.").arg(fn));
-        return;
+            return;
+#ifdef NSM
+        }
+#endif
     }
 
     clear();
@@ -1019,13 +1077,19 @@ bool MainWindow::isModified()
 
 void MainWindow::updateTempo(int p_tempo)
 {
-    engine->setTempo(p_tempo);
+    if (!midiClockAction->isChecked())
+        engine->setTempo(p_tempo);
+}
+
+void MainWindow::displayTempo(double p_tempo)
+{
+    tempoSpin->setValue(p_tempo);
 }
 
 void MainWindow::updateTransportStatus(bool on)
 {
     engine->setStatus(on);
-    tempoSpin->setDisabled(on);
+    //if (alsaMidi) tempoSpin->setDisabled(on);
 }
 
 void MainWindow::resetTransport()
@@ -1219,9 +1283,14 @@ void MainWindow::setupRecentFilesMenu()
         for (; it != recentFiles.end(); ++it) {
             fileRecentlyOpenedFiles->addAction(*it);
         }
-    } else {
+    }
+    else {
         fileRecentlyOpenedFiles->setEnabled(false);
     }
+#ifdef NSM
+    if (nsm && nsm_is_active(nsm))
+      fileRecentlyOpenedFiles->setEnabled(false);
+#endif
 }
 
 void MainWindow::recentFileActivated(QAction *action)
@@ -1291,7 +1360,16 @@ void MainWindow::checkIfFirstModule()
         if (alsaMidi) midiClockAction->setEnabled(true);
         jackSyncAction->setEnabled(true);
         fileSaveAction->setEnabled(true);
-        fileSaveAsAction->setEnabled(true);
+#ifdef NSM
+        if (nsm && nsm_is_active(nsm)) {
+            fileSaveAsAction->setDisabled(true);
+        }
+        else {
+#endif
+            fileSaveAsAction->setEnabled(true);
+#ifdef NSM
+        }
+#endif
         showAllIOAction->setEnabled(true);
         hideAllIOAction->setEnabled(true);
         runAction->setEnabled(!(midiClockAction->isChecked()
@@ -1340,6 +1418,12 @@ bool MainWindow::installSignalHandlers()
         qWarning("sigaction() failed: %s", std::strerror(errno));
         return false;
     }
+#ifdef NSM
+    if (sigaction(SIGTERM, &action, NULL) == -1) {
+        qWarning("sigaction() failed: %s", std::strerror(errno));
+        return false;
+    }
+#endif
 
     return true;
 }
@@ -1359,6 +1443,9 @@ void MainWindow::signalAction(int fd)
             break;
 
         case SIGINT:
+#ifdef NSM
+        case SIGTERM:
+#endif
             close();
             break;
 
@@ -1408,3 +1495,39 @@ void MainWindow::ftb_update_orientation(Qt::Orientation orient)
         fileToolBar->setMinimumHeight(0);
     }
 }
+
+#ifdef NSM
+int MainWindow::cb_nsm_open(const char *name, const char *display_name, const char *client_id, char **out_msg, void *userdata)
+{
+    return ((MainWindow *)userdata)->nsm_open(name, display_name, client_id, out_msg);
+}
+
+int MainWindow::cb_nsm_save ( char **out_msg, void *userdata )
+{
+    return ((MainWindow *)userdata)->nsm_save(out_msg);
+}
+
+int MainWindow::nsm_open(const char *name, const char *display_name, const char *client_id, char **out_msg)
+{
+    (void)out_msg;
+    (void)display_name;
+
+    QString configFile = name;
+    if (!alsaMidi) {
+        engine->driver->callJack(-1);
+        engine->driver->callJack(engine->getPortCount(), client_id);
+    }
+    configFile.append(".qmax");
+    emit nsmOpenFile(configFile);
+    return ERR_OK;
+}
+
+int MainWindow::nsm_save(char **out_msg)
+{
+    (void)out_msg;
+
+    int err = ERR_OK;
+    if (!saveFile()) err = ERR_GENERAL;
+    return err;
+}
+#endif
