@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -45,6 +45,10 @@
 #include "groovewidget.h"
 #include "config.h"
 
+#ifdef NSM
+#include "nsm.h"
+#endif
+
 #ifdef JACK_SESSION
 #include <jack/session.h>
 #endif
@@ -52,10 +56,12 @@
 
 static const char ABOUTMSG[] =
             "<html> <p><b><big>" APP_NAME " " PACKAGE_VERSION "</big></b></p>"
-            "<p>(C) 2009-2012 Frank Kober<br/>"
+            "<p>(C) 2009-2013 Frank Kober<br/>"
             "(C) 2011 Nedko Arnaudov<br/>"
             "(C) 2009 Guido Scholz<br/>"
             "(C) 2002-2003 Matthias Nagorni (SuSE AG Nuremberg)<br/></p>"
+            "<p><b>Contributions</b><br/>"
+            "Roy Vegard Ovesen (work on nsm support)<br/></p>"
             "<p><b>Translations</b><br/>"
             "Pavel Fric<br/>"
             "Pedro Lopez-Cabanillas<br/>"
@@ -103,6 +109,9 @@ class MainWindow : public QMainWindow
     QAction *showAllIOAction, *hideAllIOAction;
     QMenu* fileRecentlyOpenedFiles;
 
+#ifdef NSM
+    static nsm_client_t *nsm;
+#endif
 /*!
 * @brief  opens a file dialog and calls either
 * MainWindow::openTextFile or
@@ -331,27 +340,17 @@ class MainWindow : public QMainWindow
 /*!
 * @param p_portCount Number of registered MIDI output ports
 */
-    MainWindow(int p_portCount, bool p_alsamidi);
+    MainWindow(int p_portCount, bool p_alsamidi, char *execName);
     ~MainWindow();
 
     bool jackFailed;
-/*!
-* @brief  opens a QMidiArp XML session file for reading
-* using QXmlStreamReader.
-*
-* It queries XML block elements and calls the block readers
-* MainWindow::readFilePartGlobal, MainWindow::readFilePartModules,
-* MainWindow::readFilePartGUI. It sets MainWindow::lastDir according to
-* the file path given with fn and calls MainWindow::updateWindowTitle.
-* It updates MainWindow::recentFiles list.
-*
-* @param fn File name to open including its absolute path
-*/
-    void openFile(const QString&);
 
 /* SIGNALS */
   signals:
     void newTempo(int);
+#ifdef NSM
+    void nsmOpenFile(const QString & name);
+#endif
 
 /* PUBLIC SLOTS */
   public slots:
@@ -370,6 +369,19 @@ class MainWindow : public QMainWindow
 */
     void fileOpen();
 /*!
+* @brief  opens a QMidiArp XML session file for reading
+* using QXmlStreamReader.
+*
+* It queries XML block elements and calls the block readers
+* MainWindow::readFilePartGlobal, MainWindow::readFilePartModules,
+* MainWindow::readFilePartGUI. It sets MainWindow::lastDir according to
+* the file path given with fn and calls MainWindow::updateWindowTitle.
+* It updates MainWindow::recentFiles list.
+*
+* @param fn File name to open including its absolute path
+*/
+    void openFile(const QString&);
+/*!
 * @brief Slot for file Save GUI elements.
 *
 * This function calls either
@@ -454,6 +466,13 @@ class MainWindow : public QMainWindow
 *
 */
     void updateTempo(int tempo);
+/*! @brief Slot for Engine::tempoUpdated() signal.
+*
+* This function displays a new tempo value in the tempo spin box.
+* @param tempo The new tempo to be displayed
+*
+*/
+    void displayTempo(double p_tempo);
 /*! @brief Slot for MainWindow::runAction ToolButton.
 * This function calls Engine::setStatus() and disables the
 * MainWindow::tempoSpin box
@@ -536,6 +555,14 @@ class MainWindow : public QMainWindow
     void jsAction(int ev);
     void ctb_update_orientation(Qt::Orientation orient);
     void ftb_update_orientation(Qt::Orientation orient);
+
+#ifdef NSM
+    static int cb_nsm_open(const char *name, const char *display_name, const char *client_id, char **out_msg, void *userdata);
+    static int cb_nsm_save(char **out_msg, void *userdata);
+
+    int nsm_open(const char *name, const char *display_name, const char *client_id, char **out_msg);
+    int nsm_save(char **out_msg);
+#endif
 };
 
 #endif
