@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -131,6 +131,9 @@ MidiArp::MidiArp()
     grooveLength = 0;
     repeatPatternThroughChord = 1;
     isMuted = false;
+    isMutedDefer = false;
+    deferChanges = false;
+    parChangesPending = false;
     attack_time = 0.0;
     release_time = 0.0;
     sustain = false;
@@ -143,6 +146,8 @@ MidiArp::MidiArp()
 
     returnNote.resize(128);
     returnVelocity.resize(128);
+
+    needsGUIUpdate = false;
 }
 
 MidiArp::~MidiArp(){
@@ -150,7 +155,12 @@ MidiArp::~MidiArp(){
 
 void MidiArp::setMuted(bool on)
 {
-    isMuted = on;
+    isMutedDefer = on;
+    if (deferChanges) {
+        parChangesPending = true;
+    }
+    else isMuted = on;
+    needsGUIUpdate = false;
 }
 
 bool MidiArp::handleEvent(MidiEvent inEv, int tick, int keep_rel)
@@ -503,9 +513,13 @@ bool MidiArp::advancePatternIndex(bool reset)
     if (patternLen) {
         patternIndex++;
     }
+
     if ((patternIndex >= patternLen) || reset) {
         patternIndex = 0;
         restartFlag = false;
+        applyPendingParChanges();
+
+
         switch (repeatPatternThroughChord) {
             case 1:
                 noteOfs++;
@@ -816,3 +830,24 @@ void MidiArp::newGrooveValues(int p_grooveTick, int p_grooveVelocity,
     grooveVelocity = p_grooveVelocity;
     grooveLength = p_grooveLength;
 }
+
+void MidiArp::applyPendingParChanges()
+{
+    if (!parChangesPending) return;
+
+    int olddefer = deferChanges;
+    deferChanges = false;
+    setMuted(isMutedDefer);
+    deferChanges = olddefer;
+    parChangesPending = false;
+    needsGUIUpdate = true;
+}
+
+void MidiArp::setNextTick(int tick)
+{
+    returnTick = tick / (int)(nSteps*TPQN) * (int)(nSteps*TPQN);
+    patternIndex = 0;
+    currentNoteTick = returnTick;
+    arpTick = returnTick;
+    nextTick = returnTick;
+}
