@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -54,12 +54,15 @@ MidiSeq::MidiSeq()
     chIn = 0;
     queueTempo = 100.0;
     vel = 0;
+    velDefer = 0;
     transp = 0;
+    transpDefer = 0;
     size = 4;
     res = 4;
     nPoints = 16;
     maxNPoints = 16;
     notelength = 74;
+    notelengthDefer = 74;
     portOut = 0;
     channelOut = 0;
     currentIndex = 0;
@@ -69,7 +72,11 @@ MidiSeq::MidiSeq()
     grooveVelocity = 0;
     grooveLength = 0;
     isMuted = false;
+    isMutedDefer = false;
+    deferChanges = false;
+    parChangesPending = false;
     dataChanged = false;
+    needsGUIUpdate = false;
 
     int lt = 0;
     int l1 = 0;
@@ -92,7 +99,13 @@ MidiSeq::~MidiSeq(){
 
 void MidiSeq::setMuted(bool on)
 {
-    isMuted = on;
+    isMutedDefer = on;
+    if (deferChanges) {
+        parChangesPending = true;
+    }
+    else isMuted = on;
+
+    needsGUIUpdate = false;
 }
 
 bool MidiSeq::handleEvent(MidiEvent inEv, int tick)
@@ -107,9 +120,15 @@ bool MidiSeq::handleEvent(MidiEvent inEv, int tick)
             recordNote(inEv.data);
             return(false);
         }
-        if (enableNoteIn) updateTranspose(inEv.data - 60);
+        if (enableNoteIn) {
+            updateTranspose(inEv.data - 60);
+            needsGUIUpdate = true;
+        }
         if (restartByKbd && (!noteCount || trigLegato)) restartFlag = true;
-        if (enableVelIn) updateVelocity(inEv.value);
+        if (enableVelIn) {
+            updateVelocity(inEv.value);
+            needsGUIUpdate = true;
+        }
         seqFinished = false;
         noteCount++;
         if (trigByKbd && ((noteCount == 1) || trigLegato)) {
@@ -174,8 +193,9 @@ void MidiSeq::advancePatternIndex()
     reflect = pingpong;
 
     if (reverse) {
-        currentIndex--;
         if (!pivot) pivot = npoints;
+        if (currentIndex == pivot - 1) applyPendingParChanges();
+        currentIndex--;
         if (currentIndex == -1) {
             if (!enableLoop) seqFinished = true;
             if (reflect  || !backward) {
@@ -196,9 +216,11 @@ void MidiSeq::advancePatternIndex()
         }
     }
     else {
+        if (!currentIndex) applyPendingParChanges();
         currentIndex++;
         if (currentIndex == npoints) {
             if (!enableLoop) seqFinished = true;
+
             if (reflect || backward) {
                 reverse = true;
                 currentIndex = npoints - 1;
@@ -259,14 +281,31 @@ void MidiSeq::updateLoop(int val)
     curLoopMode = val;
 }
 
+void MidiSeq::updateNoteLength(int val)
+{
+    notelengthDefer = val;
+    if (deferChanges) {
+        parChangesPending = true;
+    }
+    else notelength = val;
+}
+
 void MidiSeq::updateVelocity(int val)
 {
-    vel = val;
+    velDefer = val;
+    if (deferChanges) {
+        parChangesPending = true;
+    }
+    else vel = val;
 }
 
 void MidiSeq::updateTranspose(int val)
 {
-    transp = val;
+    transpDefer = val;
+    if (deferChanges) {
+        parChangesPending = true;
+    }
+    else transp = val;
 }
 
 void MidiSeq::recordNote(int val)
@@ -274,6 +313,7 @@ void MidiSeq::recordNote(int val)
         setRecordedNote(val);
         currentRecStep++;
         currentRecStep %= (res * size);
+        dataChanged = true;
 }
 
 void MidiSeq::updateQueueTempo(int val)
@@ -398,3 +438,36 @@ void MidiSeq::newGrooveValues(int p_grooveTick, int p_grooveVelocity,
     grooveVelocity = p_grooveVelocity;
     grooveLength = p_grooveLength;
 }
+
+void MidiSeq::applyPendingParChanges()
+{
+    if (!parChangesPending) return;
+
+    int olddefer = deferChanges;
+    deferChanges = false;
+
+    setMuted(isMutedDefer);
+    updateNoteLength(notelengthDefer);
+    updateVelocity(velDefer);
+    updateTranspose(transpDefer);
+
+    deferChanges = olddefer;
+    parChangesPending = false;
+    needsGUIUpdate = true;
+
+}
+
+void MidiSeq::setNextTick(int tick)
+{
+    int tickres = TPQN/res;
+    int pos = (tick/tickres) % nPoints;
+
+    reverse = false;
+    if (pingpong || (loopMarker > 0)) reverse = (((tick/tickres) / nPoints) % 2);
+
+    if (backward) reverse = !reverse;
+    if (reverse) pos = nPoints - pos;
+
+    setCurrentIndex(pos);
+    nextTick = (tick/tickres) * tickres;
+}
