@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -24,11 +24,15 @@
 
 #include "main.h"
 #include "parstore.h"
+#include "storagebutton.h"
 
 #include "pixmaps/filesave.xpm"
 
-ParStore::ParStore(GlobStore *p_globStore, const QString &name, QWidget* parent):
-            QWidget(parent), globStore(p_globStore)
+ParStore::ParStore(GlobStore *p_globStore, const QString &name,
+            QAction *p_muteOutAction,
+            QAction *p_deferChangesAction,
+            QWidget* parent):
+                QWidget(parent), globStore(p_globStore)
 {
     // when temp.empty is true, restoring from that set is ignored
     temp.empty = false;
@@ -73,7 +77,32 @@ ParStore::ParStore(GlobStore *p_globStore, const QString &name, QWidget* parent)
     topButton = new QToolButton(this);
     topButton->setFont(QFont("Helvetica", 8));
     topButton->setText(name);
-    topButton->setMinimumSize(QSize(75, 30));
+    topButton->setMinimumSize(QSize(75, 10));
+
+    muteOutAction = p_muteOutAction;
+    muteOut = new QToolButton(this);
+    muteOut->setDefaultAction(muteOutAction);
+    muteOut->setFont(QFont("Helvetica", 8));
+    muteOut->setMinimumSize(QSize(10, 10));
+
+    deferChangesAction = p_deferChangesAction;
+    deferChanges = new QToolButton(this);
+    deferChanges->setDefaultAction(deferChangesAction);
+    deferChanges->setFont(QFont("Helvetica", 8));
+    deferChanges->setMinimumSize(QSize(10, 10));
+
+    QHBoxLayout *muteRowLayout = new QHBoxLayout;
+    muteRowLayout->addStretch();
+    muteRowLayout->addWidget(muteOut);
+    muteRowLayout->addWidget(deferChanges);
+    muteRowLayout->setMargin(0);
+    muteRowLayout->setSpacing(0);
+
+    QVBoxLayout *controlLayout = new QVBoxLayout;
+    controlLayout->addWidget(topButton);
+    controlLayout->addLayout(muteRowLayout);
+    controlLayout->setMargin(0);
+    controlLayout->setSpacing(0);
 
     QWidget *indicatorBox = new QWidget(this);
     QHBoxLayout *indicatorLayout = new QHBoxLayout;
@@ -84,12 +113,14 @@ ParStore::ParStore(GlobStore *p_globStore, const QString &name, QWidget* parent)
     indicatorLayout->setSpacing(1);
     indicatorBox->setLayout(indicatorLayout);
 
-    QWidget *topRow = new QWidget(this);
+    QFrame *topRow = new QFrame(this);
     QHBoxLayout *topRowLayout = new QHBoxLayout;
     topRowLayout->addWidget(indicatorBox);
-    topRowLayout->addWidget(topButton);
+    topRowLayout->addLayout(controlLayout);
     topRowLayout->setSpacing(0);
     topRowLayout->setMargin(0);
+    topRow->setMinimumSize(QSize(48,46));;
+    topRow->setFrameStyle(QFrame::StyledPanel);
     topRow->setLayout(topRowLayout);
 
     QVBoxLayout *buttonLayout = new QVBoxLayout();
@@ -103,6 +134,12 @@ ParStore::ParStore(GlobStore *p_globStore, const QString &name, QWidget* parent)
     locContextMenu->addAction(storeHereAction);
     connect(storeHereAction, SIGNAL(triggered()), this, SLOT(mapStoreSignal()));
 
+    QAction *onlyPatternAction = new QAction(tr("&Act on pattern only"), this);
+    onlyPatternAction->setProperty("index", list.count());
+    onlyPatternAction->setCheckable(true);
+    locContextMenu->addAction(onlyPatternAction);
+    connect(onlyPatternAction, SIGNAL(toggled(bool)), this, SLOT(updateOnlyPattern(bool)));
+
     jumpToIndexMenu = new QMenu(tr("When finished"), this);
 
     jumpToGroup = new QActionGroup(this);
@@ -192,6 +229,7 @@ void ParStore::writeData(QXmlStreamWriter& xml)
             xml.writeTextElement("pattern", list.at(ix).pattern);
 
             xml.writeTextElement("jumpTo", QString::number(jumpToList.at(ix)));
+            xml.writeTextElement("onlyPattern", QString::number((int)onlyPatternList.at(ix)));
 
             tempArray.clear();
             l1 = 0;
@@ -222,6 +260,7 @@ void ParStore::readData(QXmlStreamReader& xml)
     int ix = 0;
     int step = 0;
     int tmpjumpto = -2;
+    int tmponlypattern = 0;
 
     while (!xml.atEnd()) {
         xml.readNext();
@@ -295,6 +334,8 @@ void ParStore::readData(QXmlStreamReader& xml)
                     temp.pattern = xml.readElementText();
                 else if (xml.name() == "jumpTo")
                     tmpjumpto = xml.readElementText().toInt();
+                else if (xml.name() == "onlyPattern")
+                    tmponlypattern = xml.readElementText().toInt();
                 else if (xml.isStartElement() && (xml.name() == "muteMask")) {
                     while (!xml.atEnd()) {
                         xml.readNext();
@@ -343,6 +384,7 @@ void ParStore::readData(QXmlStreamReader& xml)
             }
             tempToList(ix);
             updateRunOnce(ix, tmpjumpto);
+            onlyPatternList.replace(ix, tmponlypattern);
             ix++;
         }
     }
@@ -350,14 +392,14 @@ void ParStore::readData(QXmlStreamReader& xml)
 
 void ParStore::addLocation()
 {
-    QToolButton *toolButton = new QToolButton(this);
+    StorageButton *toolButton = new StorageButton(this);
     toolButton->setText(QString::number(list.count()));
-    toolButton->setFixedSize(QSize(100, 25));
     toolButton->setProperty("index", list.count());
     connect(toolButton, SIGNAL(pressed()), this, SLOT(mapRestoreSignal()));
 
     toolButton->setContextMenuPolicy(Qt::ContextMenuPolicy(Qt::CustomContextMenu));
-    connect(toolButton, SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(showLocContextMenu(const QPoint &)));
+    connect(toolButton, SIGNAL(customContextMenuRequested(const QPoint &))
+                    , this, SLOT(showLocContextMenu(const QPoint &)));
 
     jumpToIndexMenu->addAction(new QAction(QString::number(list.count()), this));
     jumpToIndexMenu->actions().last()->setActionGroup(jumpToGroup);
@@ -366,6 +408,7 @@ void ParStore::addLocation()
 
     layout()->itemAt(0)->layout()->addWidget(toolButton);
     jumpToList.append(-2);
+    onlyPatternList.append(false);
 }
 
 void ParStore::removeLocation(int ix)
@@ -379,6 +422,7 @@ void ParStore::removeLocation(int ix)
     delete button;
     delete action;
     jumpToList.removeAt(ix);
+    onlyPatternList.removeAt(ix);
 
     for (int l1 = 0; l1 < jumpToList.count(); l1++) {
         if (jumpToList.at(l1) >= jumpToList.count()) updateRunOnce(l1, -2);
@@ -407,20 +451,19 @@ void ParStore::updateRunOnce(int location, int choice)
     if (choice == -2) { //stay here
         jumpToList.replace(location, -2);
         setBGColorAt(location + 1, 0);
-        ((QToolButton *)(layout()->itemAt(0)->layout()->itemAt(location + 1)
-            ->widget()))->setText(QString::number(location + 1));
+        ((StorageButton *)(layout()->itemAt(0)->layout()->itemAt(location + 1)
+            ->widget()))->setSecondText("", 0);
     }
     else if (choice == -1) { //jump back to last
         jumpToList.replace(location, -1);
         setBGColorAt(location + 1, 3);
-        ((QToolButton *)(layout()->itemAt(0)->layout()->itemAt(location + 1)
-            ->widget()))->setText(QString::number(location + 1) + " < ");
+        ((StorageButton *)(layout()->itemAt(0)->layout()->itemAt(location + 1)
+            ->widget()))->setSecondText("<- ", 1);
     }
     else if (choice >= 0) { //jump to location
         jumpToList.replace(location, choice);
-        ((QToolButton *)(layout()->itemAt(0)->layout()->itemAt(location + 1)
-            ->widget()))->setText(QString::number(location + 1)
-                    + " > " + QString::number(choice + 1));
+        ((StorageButton *)(layout()->itemAt(0)->layout()->itemAt(location + 1)
+            ->widget()))->setSecondText("-> "+QString::number(choice + 1), 2);
         setBGColorAt(location + 1, 3);
     }
 }
@@ -551,6 +594,11 @@ void ParStore::showLocContextMenu(const QPoint &pos)
 
     jumpToGroup->actions().at(jumpToList.at(senderlocation) + 2)
             ->setChecked(true);
-
+    locContextMenu->actions().at(1)->setChecked(onlyPatternList.at(senderlocation));
     locContextMenu->popup(QWidget::mapToGlobal(pos));
 }
+
+void ParStore::updateOnlyPattern(bool on)
+{
+    onlyPatternList.replace(sender()->property("index").toInt() - 1, on);
+}
