@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -69,13 +69,22 @@ class ParStore : public QWidget
   Q_OBJECT
 
   public:
-    ParStore(GlobStore *p_globStore, const QString &name = "", QWidget* parent = 0);
+    ParStore(GlobStore *p_globStore, const QString &name = "",
+                QAction* p_muteOutAction = 0,
+                QAction* p_deferChangesAction = 0,
+                QWidget* parent = 0);
     ~ParStore();
     GlobStore *globStore;
     QToolButton *topButton;
+    QAction *muteOutAction;
+    QAction *deferChangesAction;
+    QToolButton *muteOut;
+    QToolButton *deferChanges;
     Indicator *ndc;
     QList<int> jumpToList; /**< List of jumpTo configurations for each location
                             @see ParStore::updateRunOnce()*/
+    QList<bool> onlyPatternList; /**< List of switch configuration for each location
+                            @see ParStore::updateOnlyPattern()*/
     QMenu *locContextMenu;
     QMenu *jumpToIndexMenu;
     QActionGroup *jumpToGroup;
@@ -292,5 +301,6 @@ class ParStore : public QWidget
 * coordinates
 */
     void showLocContextMenu(const QPoint &pos);
+    void updateOnlyPattern(bool on);
 };
 #endif
