@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -89,7 +89,6 @@ SeqDriver::SeqDriver(
 
     lastSchedTick = 0;
     m_current_tick = 0;
-    jackOffsetTick = 0;
 
     queueStatus = false;
     startQueue = false;
@@ -97,10 +96,16 @@ SeqDriver::SeqDriver(
     useMidiClock = false;
     midiTick = 0;
     lastRatioTick = 0;
-
+    tempoChangeTick = 0;
+    midiTempoRefreshTick = 0;
+    trStartingTick = 0;
+    trLoopingTick = 0;
+    tempoChangeFrame = 0;
+    requestedTempo = 120;
     internalTempo = 120;
     initTempo();
     clockRatio = 60e9/TPQN/tempo;
+    tempoChangeTime = 0;
 
     threadAbort = false;
     start(Priority(6));
@@ -120,6 +125,7 @@ void SeqDriver::run()
 {
     snd_seq_event_t *evIn;
     bool unmatched = true;
+    double tmpTime = 0;
     MidiEvent inEv;
     int pollr = 0;
 
@@ -135,6 +141,8 @@ void SeqDriver::run()
         pollr = poll(pfds, nfds, 200);
         while (pollr > 0) {
 
+            tmpTime = getCurrentTime();
+
             snd_seq_event_input(seq_handle, &evIn);
 
             inEv.type = evIn->type;
@@ -143,35 +151,13 @@ void SeqDriver::run()
             inEv.value = 0;
 
             if ((inEv.type == EV_CLOCK)&& useMidiClock) {
+                calcCurrentTick(tmpTime);
                 midiTick++;
-                m_current_tick = midiTick*TPQN/MIDICLK_TPQN;
             }
             if (((inEv.type == EV_ECHO) || startQueue) && queueStatus) {
                 startQueue = false;
                 unmatched = false;
-                realTime = evIn->time.time;
-                if (useMidiClock) {
-                    m_current_tick = midiTick*TPQN/MIDICLK_TPQN;
-                    if (midiTick > lastRatioTick + 3 || !midiTick) {
-                        calcClockRatio();
-                        lastRatioTick = midiTick;
-                    }
-                }
-                else if (useJackSync) {
-                    jPos = jackSync->getCurrentPos();
-                    if (jPos.beats_per_minute > 0)
-                        tempo = jPos.beats_per_minute;
-
-                    m_current_tick = (long)jPos.frame * TPQN
-                            / jPos.frame_rate * tempo / 60.
-                            - jackOffsetTick;
-                    calcClockRatio();
-                }
-                else {
-                    clockRatio = 60e9/TPQN/tempo;
-                    m_current_tick = deltaToTick(aTimeToDelta(&realTime));
-                }
-
+                calcCurrentTick(aTimeToDelta(&evIn->time.time));
                 tick_callback((inEv.data));
             }
             else {
@@ -179,7 +165,6 @@ void SeqDriver::run()
                 inEv.channel = evIn->data.control.channel;
 
                 if ((inEv.type == EV_NOTEON) || (inEv.type == EV_NOTEOFF)) {
-                    realTime = evIn->time.time;
                     inEv.data = evIn->data.note.note;
                     if (inEv.type == EV_NOTEON) {
                         inEv.value = evIn->data.note.velocity;
@@ -188,15 +173,13 @@ void SeqDriver::run()
                         inEv.value = 0;
                         inEv.type = EV_NOTEON;
                     }
-                getTime();
-                m_current_tick = deltaToTick(aTimeToDelta(&tmpTime));
+                    calcCurrentTick(tmpTime);
                 }
                 else inEv.value = evIn->data.control.value;
 
                 if (inEv.type == EV_CONTROLLER) {
                     inEv.data = evIn->data.control.param;
-                getTime();
-                m_current_tick = deltaToTick(aTimeToDelta(&tmpTime));
+                    calcCurrentTick(tmpTime);
                 }
 
                 unmatched = midi_event_received(inEv);
@@ -214,6 +197,45 @@ void SeqDriver::run()
     }
 }
 
+void SeqDriver::calcCurrentTick(double tmpTime) {
+
+    if (useMidiClock) {
+        m_current_tick = midiTick * TPQN / MIDICLK_TPQN;
+        if (midiTick > lastRatioTick + 4 || !midiTick) {
+            calcClockRatio(tmpTime);
+            lastRatioTick = midiTick;
+            internalTempo = (60e9/TPQN/clockRatio);
+        }
+        if (midiTick > midiTempoRefreshTick + 48) {
+            tempoChangeTick = m_current_tick;
+            tempoChangeTime = tmpTime;
+            midiTempoRefreshTick = midiTick;
+            jackSync->tempoCb(internalTempo, jackSync->cbContext);
+        }
+    }
+    else if (useJackSync) {
+        jPos = jackSync->getCurrentPos();
+        if (jPos.beats_per_minute > 0) requestedTempo = jPos.beats_per_minute;
+
+        m_current_tick = (uint64_t)(jPos.frame - tempoChangeFrame) * TPQN  * tempo
+                / jPos.frame_rate / 60.
+                + tempoChangeTick;
+
+        tmpTime = tickToDelta(m_current_tick);
+        snd_seq_event_t ev;
+        snd_seq_ev_clear(&ev);
+        snd_seq_ev_set_queue_pos_real(&ev, queue_id, deltaToATime(tmpTime));
+        snd_seq_ev_set_direct(&ev);
+        snd_seq_event_output_direct(seq_handle, &ev);
+
+        clockRatio = 60e9/TPQN/tempo;
+    }
+    else {
+        m_current_tick = deltaToTick(tmpTime);
+        clockRatio = 60e9/TPQN/tempo;
+    }
+}
+
 void SeqDriver::initTempo()
 {
     if (useJackSync) {
@@ -223,23 +245,24 @@ void SeqDriver::initTempo()
         else
             tempo = internalTempo;
 
-        jackOffsetTick = (uint64_t)jPos.frame * TPQN
-                * tempo / (jPos.frame_rate * 60);
-        clockRatio = 60e9/TPQN/tempo;
+        tempoChangeFrame = (uint64_t)jPos.frame;
     }
     else {
         tempo = internalTempo;
-        clockRatio = 60e9/TPQN/tempo;
     }
+
+    clockRatio = 60e9/TPQN/tempo;
+
     if (useMidiClock) {
         midiTick = 0;
         lastRatioTick = 0;
+        midiTempoRefreshTick = 0;
     }
 }
 
 void SeqDriver::sendMidiEvent(MidiEvent outEv, int n_tick, unsigned outport, unsigned length)
 {
-  //~ qWarning("sendMidiEvent([%d, %d, %d, %d], %u, %u) at tick %d", ev.type, ev.channel, ev.data, ev.value, outport, duration, n_tick);
+    //~ qWarning("sendMidiEvent([%d, %d, %d, %d], %u, %u) at tick %d", ev.type, ev.channel, ev.data, ev.value, outport, duration, n_tick);
     snd_seq_event_t ev;
     snd_seq_ev_clear(&ev);
 
@@ -274,14 +297,27 @@ bool SeqDriver::requestEchoAt(int echo_tick, bool echo_from_trig)
     return true;
 }
 
-void SeqDriver::setTempo(int bpm)
+void SeqDriver::requestTempo(double bpm)
 {
-    tempo = bpm;
+    double tmpTime = getCurrentTime();
+    calcCurrentTick(tmpTime);
+    tempoChangeTick = m_current_tick;
+    tempoChangeTime = tmpTime;
     internalTempo = bpm;
-    clockRatio = 60e9/TPQN/tempo;
+    initTempo();
+    requestedTempo = bpm;
+    requestEchoAt(lastSchedTick + 1);
 }
 
-void SeqDriver::getTime()
+void SeqDriver::setTempo(double bpm)
+{
+    tempoChangeTick = m_current_tick;
+    tempoChangeTime = getCurrentTime();
+    internalTempo = bpm;
+    initTempo();
+}
+
+double SeqDriver::getCurrentTime()
 {
     snd_seq_queue_status_t *status;
 
@@ -290,8 +326,10 @@ void SeqDriver::getTime()
 
     const snd_seq_real_time_t* current_time =
         snd_seq_queue_status_get_real_time(status);
-    tmpTime = *current_time;
+    snd_seq_real_time_t tmpTime = *current_time;
     snd_seq_queue_status_free(status);
+
+    return aTimeToDelta(&tmpTime);
 }
 
 void SeqDriver::setTransportStatus(bool run)
@@ -301,12 +339,16 @@ void SeqDriver::setTransportStatus(bool run)
         startQueue = true;
 
         initTempo();
-
+        tempoChangeTick = 0;
+        tempoChangeTime = 0;
+        tempoChangeFrame = 0;
+        if (useJackSync)
+            trStartingTick = jackSync->trStartingTick;
+        else
+            trStartingTick = 0;
         snd_seq_start_queue(seq_handle, queue_id, NULL);
         snd_seq_drain_output(seq_handle);
-
-        requestEchoAt(0);
-
+        calcCurrentTick(0);
         printf("Alsa Queue started \n");
     }
     else {
@@ -328,16 +370,23 @@ void SeqDriver::setUseMidiClock(bool on)
 {
     clockRatio = 60e9/TPQN/tempo;
     useMidiClock = on;
+    if (!on) jackSync->tempoCb(internalTempo, jackSync->cbContext);
 }
 
 double SeqDriver::tickToDelta(int tick)
 {
-    return (double)clockRatio * tick;
+    if (tick > tempoChangeTick)
+        return (double)clockRatio * (tick-tempoChangeTick) + tempoChangeTime;
+    else
+        return 0;
 }
 
 int SeqDriver::deltaToTick(double curtime)
 {
-    return (int)(curtime / clockRatio);
+    if (tempoChangeTime < curtime)
+        return (int)((curtime-tempoChangeTime) / clockRatio)+tempoChangeTick;
+    else
+        return tempoChangeTick;
 }
 
 double SeqDriver::aTimeToDelta(snd_seq_real_time_t* atime)
@@ -347,23 +396,23 @@ double SeqDriver::aTimeToDelta(snd_seq_real_time_t* atime)
 
 const snd_seq_real_time_t* SeqDriver::deltaToATime(double curtime)
 {
-    delta.tv_sec = (int)(curtime * 1e-9);
-    delta.tv_nsec = (long)(curtime - (double)delta.tv_sec * 1e9);
-    return &delta;
+    atime.tv_sec = (int)(curtime * 1e-9);
+    atime.tv_nsec = (uint64_t)(curtime - (double)atime.tv_sec * 1e9);
+    return &atime;
 }
 
-void SeqDriver::calcClockRatio()
+void SeqDriver::calcClockRatio(double realtime)
 {
     double old_clock_ratio = clockRatio;
 
     if (m_current_tick > 0) {
-        clockRatio = aTimeToDelta(&realTime)/m_current_tick;
+        clockRatio = (realtime - tempoChangeTime)/(m_current_tick - tempoChangeTick);
     }
     if ((clockRatio == 0) || (clockRatio > 60e9 / tempo)) {
         clockRatio = old_clock_ratio;
     }
-    clockRatio += old_clock_ratio;
-    clockRatio *= .5;
+    clockRatio+=old_clock_ratio;
+    clockRatio*=.5;
 }
 
 int SeqDriver::getClientId()
@@ -388,7 +437,7 @@ void SeqDriver::setUseJackTransport(bool on)
     }
 }
 
-bool SeqDriver::callJack(int portcount)
+bool SeqDriver::callJack(int portcount, const QString & clientname)
 {
-    return jackSync->callJack(portcount);
+    return jackSync->callJack(portcount, clientname);
 }
