@@ -4,7 +4,7 @@
  *
  * @section LICENSE
  *
- *      Copyright 2009, 2010, 2011, 2012 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2013 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -44,13 +44,6 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, GlobStore *p_globStore,
 
     midiControl = new MidiControl(this);
 
-    parStore = new ParStore(globStore, name, this);
-    midiControl->addMidiLearnMenu("Restore_"+name, parStore->topButton, 7);
-    connect(parStore, SIGNAL(store(int, bool)),
-             this, SLOT(storeParams(int, bool)));
-    connect(parStore, SIGNAL(restore(int)),
-             this, SLOT(restoreParams(int)));
-
     manageBox = new ManageBox("Seq:", true, this);
 
     // Display group box on right
@@ -230,13 +223,24 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, GlobStore *p_globStore,
 
     cursor = new Cursor('S', this);
 
-    muteOut = new QPushButton(tr("&Mute"),this);
+    muteOutAction = new QAction(tr("&Mute"),this);
+    muteOutAction->setCheckable(true);
+    connect(muteOutAction, SIGNAL(toggled(bool)), this, SLOT(setMuted(bool)));
+    muteOut = new QToolButton(this);
+    muteOut->setDefaultAction(muteOutAction);
     muteOut->setFont(QFont("Helvetica", 8));
-    muteOut->setMinimumSize(QSize(35,10));
-    muteOut->setCheckable(true);
-    connect(muteOut, SIGNAL(toggled(bool)), this, SLOT(setMuted(bool)));
+    muteOut->setMinimumSize(QSize(35,20));
     midiControl->addMidiLearnMenu("MuteToggle", muteOut, 0);
 
+    deferChangesAction = new QAction("D", this);
+    deferChangesAction->setToolTip(tr("Defer mute, velocity, note length and transpose to pattern end"));
+    deferChangesAction->setCheckable(true);
+    connect(deferChangesAction, SIGNAL(toggled(bool)), this, SLOT(updateDeferChanges(bool)));
+
+    QToolButton *deferChangesButton = new QToolButton(this);
+    deferChangesButton->setDefaultAction(deferChangesAction);
+    deferChangesButton->setFixedSize(20, 20);
+
     loopBox = new QComboBox(seqBox);
     QStringList names;
     names.clear();
@@ -321,7 +325,8 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, GlobStore *p_globStore,
 
     QGridLayout *paramBoxLayout = new QGridLayout;
     paramBoxLayout->addWidget(loopBox, 0, 0, 1, 2);
-    paramBoxLayout->addWidget(muteOut, 1, 0, 1, 2);
+    paramBoxLayout->addWidget(muteOut, 1, 0, 1, 1);
+    paramBoxLayout->addWidget(deferChangesButton, 1, 1, 1, 2);
     paramBoxLayout->addWidget(recordButtonLabel, 2, 0);
     paramBoxLayout->addWidget(recordButton, 2, 1);
     paramBoxLayout->addWidget(resBoxLabel, 3, 0);
@@ -341,13 +346,20 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, GlobStore *p_globStore,
     }
     seqBox->setLayout(seqBoxLayout);
 
-    muteOut->setChecked(mutedAdd);
-
     QHBoxLayout *widgetLayout = new QHBoxLayout;
     widgetLayout->addWidget(seqBox, 1);
     widgetLayout->addWidget(hideInOutBoxButton, 0);
     widgetLayout->addWidget(inOutBox, 0);
 
+    parStore = new ParStore(globStore, name, muteOutAction, deferChangesAction, this);
+    midiControl->addMidiLearnMenu("Restore_"+name, parStore->topButton, 7);
+    connect(parStore, SIGNAL(store(int, bool)),
+             this, SLOT(storeParams(int, bool)));
+    connect(parStore, SIGNAL(restore(int)),
+             this, SLOT(restoreParams(int)));
+
+    muteOutAction->setChecked(mutedAdd);
+
     setLayout(widgetLayout);
     recordMode = false;
     dataChanged=false;
@@ -401,6 +413,8 @@ void SeqWidget::writeData(QXmlStreamWriter& xml)
         xml.writeStartElement("output");
             xml.writeTextElement("muted", QString::number(
                 midiWorker->isMuted));
+            xml.writeTextElement("defer", QString::number(
+                midiWorker->deferChanges));
             xml.writeTextElement("port", QString::number(
                 midiWorker->portOut));
             xml.writeTextElement("channel", QString::number(
@@ -503,7 +517,9 @@ void SeqWidget::readData(QXmlStreamReader& xml)
                 if (xml.isEndElement())
                     break;
                 if (xml.name() == "muted")
-                    muteOut->setChecked(xml.readElementText().toInt());
+                    muteOutAction->setChecked(xml.readElementText().toInt());
+                else if (xml.name() == "defer")
+                    deferChangesAction->setChecked(xml.readElementText().toInt());
                 else if (xml.name() == "channel") {
                     tmp = xml.readElementText().toInt();
                     channelOut->setCurrentIndex(tmp);
@@ -670,7 +686,7 @@ void SeqWidget::updateTrigLegato(bool on)
 
 void SeqWidget::updateNoteLength(int val)
 {
-    midiWorker->notelength = val + val;
+    midiWorker->updateNoteLength(val + val);
     modified = true;
 }
 
@@ -724,29 +740,16 @@ void SeqWidget::updateLoop(int val)
 
 void SeqWidget::updateVelocity(int val)
 {
-    midiWorker->vel = val;
+    midiWorker->updateVelocity(val);
     modified = true;
 }
 
 void SeqWidget::updateTranspose(int val)
 {
-    midiWorker->transp = val;
+    midiWorker->updateTranspose(val);
     modified = true;
 }
 
-void SeqWidget::processNote(int note, int vel)
-{
-    if (!recordMode) {
-        if (enableNoteIn->isChecked()) updateTranspose(note - 60);
-        if (enableVelIn->isChecked()) updateVelocity(vel);
-        needsGUIUpdate = true;
-    }
-    else {
-        screen->setCurrentRecStep(midiWorker->currentRecStep);
-        dataChanged=true;
-    }
-}
-
 void SeqWidget::mouseMoved(double mouseX, double mouseY, int buttons)
 {
     if ((mouseX > 1) || (mouseX < 0) || (mouseY > 1)) return;
@@ -795,7 +798,15 @@ void SeqWidget::setInOutBoxVisible(bool on)
 void SeqWidget::setMuted(bool on)
 {
     midiWorker->setMuted(on);
-    screen->setMuted(on);
+    screen->setMuted(midiWorker->isMuted);
+    parStore->ndc->setMuted(midiWorker->isMuted);
+    modified = true;
+}
+
+void SeqWidget::updateDeferChanges(bool on)
+{
+    midiWorker->updateDeferChanges(on);
+    modified = true;
 }
 
 void SeqWidget::setPortOut(int value)
@@ -897,6 +908,7 @@ void SeqWidget::storeParams(int ix, bool empty)
 
 void SeqWidget::restoreParams(int ix)
 {
+    midiWorker->applyPendingParChanges();
     if (parStore->list.at(ix).empty) return;
     for (int l1 = 0; l1 < parStore->list.at(ix).wave.count(); l1++) {
         midiWorker->customWave.replace(l1, parStore->list.at(ix).wave.at(l1));
@@ -911,22 +923,25 @@ void SeqWidget::restoreParams(int ix)
 
     resBox->setCurrentIndex(parStore->list.at(ix).res);
     loopBox->setCurrentIndex(parStore->list.at(ix).loopMode);
-    notelength->setValue(parStore->list.at(ix).notelen);
-    transpose->setValue(parStore->list.at(ix).transp);
-    velocity->setValue(parStore->list.at(ix).vel);
-    setDispVert(parStore->list.at(ix).dispVertical);
+    if (!parStore->onlyPatternList.at(ix)) {
+        midiWorker->notelength = parStore->list.at(ix).notelen;
+        midiWorker->transp = parStore->list.at(ix).transp;
+        midiWorker->vel = parStore->list.at(ix).vel;
+        setDispVert(parStore->list.at(ix).dispVertical);
 
-    //muteOut->setChecked(parStore->list.at(ix).muteOut);
-    chIn->setCurrentIndex(parStore->list.at(ix).chIn);
-    updateChIn(parStore->list.at(ix).chIn);
-    channelOut->setCurrentIndex(parStore->list.at(ix).channelOut);
-    updateChannelOut(parStore->list.at(ix).channelOut);
-    setPortOut(parStore->list.at(ix).portOut);
-    updatePortOut(parStore->list.at(ix).portOut);
+        //muteOut->setChecked(parStore->list.at(ix).muteOut);
+        chIn->setCurrentIndex(parStore->list.at(ix).chIn);
+        updateChIn(parStore->list.at(ix).chIn);
+        channelOut->setCurrentIndex(parStore->list.at(ix).channelOut);
+        updateChannelOut(parStore->list.at(ix).channelOut);
+        setPortOut(parStore->list.at(ix).portOut);
+        updatePortOut(parStore->list.at(ix).portOut);
+    }
     updateLoop(parStore->list.at(ix).loopMode);
-
     updateWaveForm(parStore->list.at(ix).waveForm);
     midiWorker->setCurrentIndex(0);
+
+    needsGUIUpdate = true;
 }
 
 void SeqWidget::copyParamsFrom(SeqWidget *fromWidget)
@@ -976,7 +991,7 @@ void SeqWidget::copyParamsFrom(SeqWidget *fromWidget)
     midiWorker->setLoopMarker(tmp);
     screen->setLoopMarker(tmp);
     midiControl->setCcList(fromWidget->midiControl->ccList);
-    muteOut->setChecked(true);
+    muteOutAction->setChecked(true);
     updateWaveForm(0);
 }
 
@@ -1025,17 +1040,17 @@ void SeqWidget::handleController(int ccnumber, int channel, int value)
 
                 case 3: if (min == max) {
                             if (value == max) {
-                                m = recordMode;
-                                setRecord(!m);
+                                m = midiWorker->recordMode;
+                                midiWorker->setRecordMode(!m);
                                 return;
                             }
                         }
                         else {
                             if (value == max) {
-                                setRecord(true);
+                                midiWorker->setRecordMode(true);
                             }
                             if (value == min) {
-                                setRecord(false);
+                                midiWorker->setRecordMode(false);
                             }
                         }
                 break;
@@ -1083,19 +1098,22 @@ void SeqWidget::updateDisplay()
         midiWorker->dataChanged=false;
         midiWorker->getData(&data);
         screen->updateData(data);
+        if (recordMode) screen->setCurrentRecStep(midiWorker->currentRecStep);
         cursor->updateNumbers(midiWorker->res, midiWorker->size);
     }
     screen->updateDraw();
     cursor->updateDraw();
     midiControl->update();
 
-    if (!needsGUIUpdate) return;
+    if (!(needsGUIUpdate || midiWorker->needsGUIUpdate)) return;
 
     transpose->setValue(midiWorker->transp);
     notelength->setValue(midiWorker->notelength/2);
     velocity->setValue(midiWorker->vel);
-    muteOut->setChecked(midiWorker->isMuted);
-    recordAction->setChecked(recordMode);
+    muteOutAction->setChecked(midiWorker->isMuted);
+    screen->setMuted(midiWorker->isMuted);
+    parStore->ndc->setMuted(midiWorker->isMuted);
+    recordAction->setChecked(midiWorker->recordMode);
     resBox->setCurrentIndex(resBoxIndex);
     updateRes(resBoxIndex);
     sizeBox->setCurrentIndex(sizeBoxIndex);
@@ -1103,4 +1121,5 @@ void SeqWidget::updateDisplay()
     loopBox->setCurrentIndex(midiWorker->curLoopMode);
 
     needsGUIUpdate = false;
+    midiWorker->needsGUIUpdate = false;
 }
